class PcApp extends React.Component
{
  constructor(props)
  {
    super(props);

    setInterval(()=>{PcConfig.sayAlive()}, 1000);

    this.state = {is_sane_at_startup:true, initialized:false, device:props.device, renderSize:props.device=="surface" ? props.renderSize*0.75 : props.renderSize };
    this.app = React.createRef();
    this.mouseTimeout = null;
    this.createElementsAnimationStyle();

    //pre-inits. Warning: do init reinitialize in PcConfig.initialize() 
    PcConfig.renderSize = props.device=="surface" ? props.renderSize*0.75 : props.renderSize;
    PcConfig.device = props.device;
  }

  static getDerivedStateFromError(error)
  {
    if (PcConfig.useElectron())
    {
      let remote = require('electron').remote;
      let testMode = remote.getGlobal('sharedObj').testMode;

      if (testMode)
      {
        PcDbg.log(true, "ERROR CAUGHT IN TEST MODE");
      }
      else {
        PcDbg.log(true, "ERROR CAUGHT, RESTART IN 15 SECONDS");
        setTimeout(()=>{PcConfig.restartAll()}, 15000);
      }

    }
    else {
      PcDbg.log(true, "ERROR CAUGHT IN WEB MODE");
    }

    return {hasError:true};
  }

  async onInitialized()
  { 
    PcDbg.log(CRASHDBG, "entering (generating scenario)");
    let C;   
    if (PcConfig.scenario.mode=="photobooth" || PcConfig.scenario.mode=="photokiosk" || PcConfig.scenario.mode=="cyclomaton")
    {
      C = PcScenario;
    }
    else if (PcConfig.scenario.mode=="videobooth" || PcConfig.scenario.mode=="videokiosk")
    {
      C = PcScenarioVideo;
    }
    this.scenario = e(C,
      {
        mode:PcConfig.scenario.mode,
        view:this.props.view,
        renderSize:this.state.renderSize,
        device:this.state.device,
        is_sane:this.state.is_sane_at_startup
      }
    );
      
    await PcCaptureSession.initialize(()=>{
      this.setState({initialized:true});
    });
  }

  createElementsAnimationStyle()
  {
    var sheet = document.createElement('style');
    sheet.innerHTML = "@keyframes elementMoveDownAndFadeIn {from { opacity:0; transform:translateY(" + (-5*PcConfig.renderSize) + "px)} to { opacity:1;}}";
    document.body.appendChild(sheet);
  }

  onMouseMove()
  {
    clearTimeout(this.mouseTimeout);
    this.app.current.style.cursor="auto";

    this.mouseShowTimeout = setTimeout(()=>{
      this.app.current.style.cursor="none";
    }, 2000);
  }
  
  async componentDidMount() // once, on first refresh
  {
    PcDbg.log(CRASHDBG, "entering");
    await PcConfig.initialize(null, this.props.xml, this.props.host, this.props.lang);
    var is_sane = await PcConfig.CheckCabinSanity(false);
    this.setState({is_sane_at_startup:is_sane})
    await this.onInitialized();
    PcDbg.log(CRASHDBG, "exiting (is_sane)", is_sane);
  }

  render()
  {
    if (!this.state.initialized)
    {
      return e("div",
        {
          ref:this.app,
          className:"pcapp",
          onMouseMove: ()=>this.onMouseMove(),
          style:{
            backgroundColor:"#000000",
            cursor:"none",
          }
        },
        e(PcScreenMainLoading,
          {
            active:true,
            renderSize:this.state.renderSize,
            width:document.body.clientWidth,
            height:document.body.clientHeight,
          }
        )
      );
    }
    else {


      return e("div",
        {
          ref:this.app,
          className:"pcapp",
          onMouseMove: ()=>this.onMouseMove(),
          style:{
            backgroundColor:"#000000",
            cursor:"none",
          }
        },
        this.scenario
      );
    }
  }

}
