class PcScreenIndex {

  // Object.keys(PcSceenIndex).forEach(index => console.log("index:", index))
  // Create new instances of the same class as static attributes
  static Intro  = new PcScreenIndex("intro");
  static Format = new PcScreenIndex("format");
  static Start = new PcScreenIndex("start");
  static Capture = new PcScreenIndex("capture");
  static Loading = new PcScreenIndex("loading");
  static TemplateAndFilter = new PcScreenIndex("templateAndFilter");
  static Infos = new PcScreenIndex("infos");
  static End = new PcScreenIndex("end");
  // EPT-specific screen indexes
  static OutOfOrder = new PcScreenIndex("out_of_order");
  static PaymentAsk = new PcScreenIndex("payment_ask");
  static PaymentConfirm = new PcScreenIndex("payment_confirm");

  constructor(name) {
    this.name = name;
  }
  toString() {
    return `PcScreenIndex.${this.name}`;
  }
}
class PcScreenTransition {

  constructor(prev, next)
  {
    this.prev = prev;
    this.next = next;
    //this.current = current;
  }

}

class PcScenario extends React.Component
{
  constructor(props)
  {
    super(props);

    if (PcConfig.useElectron()) {
      this.fs = require('electron').remote.require("fs");
      this.process = require('electron').remote.require("child_process");
    }

    // if the CheckCabinSanity() done in PcApp failed just before instancing PcScenario, go to PcScreenIndex.OutOfOrder;
    this.startScreen = this.props.is_sane ? PcScreenIndex.Intro : PcScreenIndex.OutOfOrder; 
    PcDbg.log(true, "startscreen is " + JSON.stringify(this.startScreen));
    this.scenario = {};
    this.sleepTimer = null;
    this.cabinSanityCheckTimer = null;
    this.sleepTimerFreezed = false;
    this.shootAttemptCount = 0;
    var screenStates = {};
    Object.keys(PcScreenIndex).forEach(index => {screenStates[PcScreenIndex[index]] = false;});

    this.screenCamera = React.createRef();
    this.screenInfos = React.createRef();

    this.currentScreenIndex = 0;

    PcCameraControl.init();

    this.state = {
      shootAttemptCount:this.shootAttemptCount,
      screenActive:screenStates,
      screenBackground:PcConfig.screen.intro.background,
      displayMainLoader:"block",
    }

    this.buildScenario();
    this.resetSleepTimer("constructor");

    setTimeout(()=>{this.setState({displayMainLoader:"none"}); this.goScreen(this.startScreen)}, 2000);
  }



  buildScenario()
  {
      this.scenario[PcScreenIndex.Intro] = new PcScreenTransition(PcScreenIndex.Intro, PcScreenIndex.Format);
      this.scenario[PcScreenIndex.Format] = new PcScreenTransition(PcScreenIndex.Intro, PcScreenIndex.Start);
      this.scenario[PcScreenIndex.Start] = new PcScreenTransition(PcScreenIndex.Format, PcScreenIndex.Capture);
      this.scenario[PcScreenIndex.Capture] = new PcScreenTransition(PcScreenIndex.Start, PcScreenIndex.Loading);
      this.scenario[PcScreenIndex.Loading] = new PcScreenTransition(PcScreenIndex.Capture, PcScreenIndex.TemplateAndFilter);
      this.scenario[PcScreenIndex.TemplateAndFilter] = new PcScreenTransition(PcScreenIndex.Format, PcScreenIndex.Infos);
      this.scenario[PcScreenIndex.Infos] = new PcScreenTransition(PcScreenIndex.TemplateAndFilter, PcScreenIndex.End);
      this.scenario[PcScreenIndex.End] = new PcScreenTransition(PcScreenIndex.Infos, PcScreenIndex.Intro);
      this.scenario[PcScreenIndex.PaymentAsk] = new PcScreenTransition(PcScreenIndex.Intro,  PcScreenIndex.PaymentConfirm);
      this.scenario[PcScreenIndex.PaymentConfirm] = new PcScreenTransition(PcScreenIndex.Format,  PcScreenIndex.Start);
      this.scenario[PcScreenIndex.OutOfOrder] = new PcScreenTransition(PcScreenIndex.OutOfOrder,  PcScreenIndex.OutOfOrder);

  } 

   
  resetSleepTimer(from=null) // at PcScenario div level
  {
    PcDbg.log(CRASHDBG, from, "with sleep timer", PcConfig.scenario.sleepTimer);
  
    if (this.sleepTimerFreezed) {
      PcDbg.log(CRASHDBG, "freezed, bye");
      return;
    } 
    this.stopSleepTimer();
    this.sleepTimer = setTimeout(()=>this.onSleepTimer(),PcConfig.scenario.sleepTimer);
  }

  stopSleepTimer()
  {
    clearTimeout(this.sleepTimer);
  }

  freezeAndStopSleepTimer()
  {
    this.stopSleepTimer();
    this.sleepTimerFreezed = true;
  }

  unfreezeSleepTimer()
  {
    this.sleepTimerFreezed = false;
  }

  onSleepTimer()
  {
    PcDbg.log(CRASHDBG, "onSleepTimer: goto " + PcScreenIndex.Intro);
    if (this.state.screenActive[PcScreenIndex.End])
    {
      window.location.reload();
    }
    else
    {
      this.goScreen(PcScreenIndex.Intro);
    }
  }

  goPrevScreen(e, screenIndex)
  {
    PcDbg.log(CRASHDBG, "enter goPrevScreen:" + screenIndex);
    var nextScreenIndex = this.scenario[screenIndex].prev;
    this.goScreen(nextScreenIndex, -1);
    PcDbg.log(CRASHDBG, "exit goPrevScreen:" + screenIndex);
  }

  goNextScreen(e, screenIndex)
  {
    PcDbg.log(CRASHDBG, "enter goNextScreen:" + screenIndex);
    var nextScreenIndex = this.scenario[screenIndex].next;
    this.goScreen(nextScreenIndex, 1);
    PcDbg.log(CRASHDBG, "exit goNextScreen:" + screenIndex);
  }

  goScreen(nextScreenIndex, increment)
  {
    PcDbg.log(CRASHDBG, "enter goScreen " + nextScreenIndex);

    if (PcConfig.useElectron()) PcConfig.electron.send('windowTop');

    this.currentScreenIndex = nextScreenIndex;

    var screenStates = {};
    var screenBackground = "";
    var endLabel = "";//PcConfig.screen.end.title;
    var endLoaded = false;
    var formKeepState = false;
    var langButtonVisible = false;

    Object.keys(PcScreenIndex).forEach(index => {screenStates[PcScreenIndex[index]] = false;});
    screenStates[nextScreenIndex] = true;

    switch (nextScreenIndex)
    {
      case PcScreenIndex.OutOfOrder:
        screenBackground = PcConfig.screen.out_of_order.background; 
        this.freezeAndStopSleepTimer();
        langButtonVisible = false;
        PcCameraControl.stopLiveView();
        PcConfig.resetMemory();
        PcCaptureSession.reset();
        break;

      case PcScreenIndex.Intro:
        screenBackground = PcConfig.screen.intro.background;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.stopSleepTimer();        
        PcCameraControl.stopLiveView();
        PcConfig.resetMemory();
        this.shootAttemptCount = 0;
        PcCaptureSession.reset();
        break;

      case PcScreenIndex.Format:
        screenBackground = PcConfig.screen.format.background;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.resetSleepTimer(nextScreenIndex);

        if (PcConfig.getMemory("currentFormat")=="")
        {
          if (! PcConfig.scenario.payment.active && PcConfig.screen.format.formats.length<=2)
          {
            PcConfig.setMemory("currentFormat", PcConfig.screen.format.formats[0]);
            PcConfig.setMemory("currentFormatIndex", 0);
          }
          else {
            let f = Math.ceil((PcConfig.screen.format.formats.length-1)/2);
            PcConfig.setMemory("currentFormat", PcConfig.screen.format.formats[f]);
            PcConfig.setMemory("currentFormatIndex", f);
          }
        }

        if (! PcConfig.scenario.payment.active && PcConfig.screen.format.formats.length<2)
        {
          increment==1 ? this.goNextScreen(null, nextScreenIndex) : this.goPrevScreen(null, nextScreenIndex);
          return;
        }

        PcCameraControl.stopLiveView();

        break;

      case PcScreenIndex.PaymentAsk:
        screenBackground = PcConfig.screen.paymentAsk.background;
        this.freezeAndStopSleepTimer(); // we rely on specific PcConfig.screen.paymentAsk.timeout to time out
        langButtonVisible = false;
        break;

      case PcScreenIndex.PaymentConfirm: 
        screenBackground = PcConfig.screen.paymentConfirm.background; 
        this.freezeAndStopSleepTimer(); // we rely on specific PcConfig.screen.paymentConfirm.timeout to time out
        langButtonVisible = false;        
        break;

      case PcScreenIndex.Start:
        screenBackground = PcConfig.screen.start.background;
        formKeepState = false;
        langButtonVisible = true;        
        if (! PcConfig.scenario.payment.active) {
          this.unfreezeSleepTimer();
          this.resetSleepTimer(nextScreenIndex);
        } // else, if payment.active, we rely on the payment mode timeout bar to timeout 
        PcCameraControl.startLiveView();
        PcConfig.setMemory('formShouldReset', true);
        PcCaptureSession.reset();
        PcCameraControl.deleteOldShoots();
        PcConfig.setMemory("currentSkinIndex", "");

        let fIndex = PcConfig.screen.templateAndFilter.defaultFilterIndex;
        if (fIndex>=PcConfig.screen.templateAndFilter.filters.length) fIndex = PcConfig.screen.templateAndFilter.filters.length-1;
        if (fIndex<0) fIndex = 0;
        PcConfig.setMemory("currentFilterIndex", fIndex);

        break;

      case PcScreenIndex.Capture:
        screenBackground = PcConfig.screen.capture.background;
        formKeepState = false;
        langButtonVisible = false;
        this.freezeAndStopSleepTimer();
        break;

      case PcScreenIndex.Loading:
        this.shootAttemptCount++;
        screenBackground = PcConfig.screen.loading.background;
        formKeepState = false;
        langButtonVisible = false;
        this.freezeAndStopSleepTimer();
        break;

      case PcScreenIndex.TemplateAndFilter:
        if (PcConfig.getMemory("winner")==1)
        {
          screenBackground = PcConfig.instantWin.background;
        }
        else {
          if (PcConfig.framingAlert.length>0)
          {
            screenBackground = PcConfig.screen.templateAndFilter.framingBackground;
          }
          else
          {
            screenBackground = PcConfig.screen.templateAndFilter.background;
          }
        }
        formKeepState = true;
        langButtonVisible = true;
        if (! PcConfig.scenario.payment.active) {
          this.unfreezeSleepTimer();
          this.resetSleepTimer(nextScreenIndex);
        } // else, if payment.active, we rely on the payment mode timeout bar to timeout 
        PcCameraControl.startLiveView();

        if (PcConfig.getMemory("currentSkinIndex")=="")
        {
          if (PcConfig.getMemory("winner")==1)
          {
            PcConfig.setMemory("currentSkin", PcConfig.instantWin.lots[PcConfig.getMemory("winnerLot")][PcConfig.getMemory("currentFormatIndex")][0]);
            PcConfig.setMemory("currentSkinIndex", 0);
          }
          else if (PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")].length<=2)
          {
            PcConfig.setMemory("currentSkin", PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")][0]);
            PcConfig.setMemory("currentSkinIndex", 0);
          }
          else {
            let s = Math.ceil((PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")].length-1)/2);
            PcConfig.setMemory("currentSkin", PcConfig.screen.templateAndFilter.skins[PcConfig.getMemory("currentFormatIndex")][s]);
            PcConfig.setMemory("currentSkinIndex", s);
          }
        }

        PcConfig.setMemory("currentFilterDict", PcConfig.screen.templateAndFilter.filters[PcConfig.getMemory("currentFilterIndex")]);
        //PcConfig.setMemory("currentFilterIndex", 0);

        break;

      case PcScreenIndex.Infos:

        if (PcConfig.screen.infos.questions.length<1)
        {
          increment==1 ? this.goNextScreen(null, nextScreenIndex) : this.goPrevScreen(null, nextScreenIndex);
          return;
        }

        screenBackground = PcConfig.screen.infos.background;
        formKeepState = true;
        langButtonVisible = true;
        if (! PcConfig.scenario.payment.active) {
          this.unfreezeSleepTimer();
          this.resetSleepTimer(nextScreenIndex);
        } // else, if payment.active, we rely on the payment mode timeout bar to timeout
        PcCameraControl.stopLiveView();
        if (PcConfig.screen.infos.listingFile!="") PcConfig.setMemory("formShouldReset", true);
        break;

      case PcScreenIndex.End:
        screenBackground = PcConfig.screen.end.background;
        formKeepState = false;
        langButtonVisible = true;
        this.unfreezeSleepTimer();
        this.resetSleepTimer(nextScreenIndex);
        endLoaded = false;
        setTimeout(async()=>await this.saveAll(()=>{
            this.setState({endLoaded:true})
          }), 100);
        break;
      default:
        var msg = `ERROR CANNOT FIND CASE FOR ${JSON.stringify(nextScreenIndex)}`;
        PcDbg.log(true, msg);
        throw new Error(msg);
    };

    if (PcConfig.scenario.languages.length<=1) langButtonVisible = false;

    this.setState( /* this is where the updated state variables are chained to the other classes props using them and refreshes their respective componentDidMount()  */
      {
        shootAttemptCount:this.shootAttemptCount,
        screenActive : screenStates,
        screenBackground : screenBackground,
        formKeepState:formKeepState,
        langButtonVisible:langButtonVisible,
        endLabel:endLabel,
        endLoaded:endLoaded,
      });
      PcDbg.log(CRASHDBG, "exit goScreen " + nextScreenIndex);
    }

  async saveAll(onSuccess)
  {
    PcDbg.log(CRASHDBG, "enter saveAll");
    var format = PcConfig.getMemory("currentFormat");
    var skin = PcConfig.getMemory("currentSkin");
    var filter_dict = PcConfig.getMemory("currentFilterDict");


    var useRemoveBg = PcConfig.getMemory("useRemoveBgChoice"); // SUPER SUPER FIXME // this is the user's final choice 
    PcDbg.log(REMOVEDBG, "entering with useRemoveBg(choice)", useRemoveBg);
    var shots;

    if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==false)
    {
      shots =PcCaptureSession.defaultShots;
    }
    else {
      shots =PcCaptureSession.shots;
    }

    var photoData = await PcTemplates.createPhotoData(format, skin, filter_dict, shots, useRemoveBg);

    if (format.animation=="gif4")
    {
      await PcTemplates.createGif4Data(format, skin, filter_dict, shots, (gifData)=>{
        PcConfig.callSaveContent(photoData, true, (id_photo)=>{ // callsaveContent MUST NOT put sync == NULL else there is a race condition. heuristic: when "animation" is true, callSaveGif will be called afterwards
          if (format.printing) this.printAll(id_photo);
          PcConfig.callSaveGif(gifData, id_photo, onSuccess);
        },);
      });
    }
    else if (format.animation=="boomerang")
    {
      await PcTemplates.createBoomerangData(format, skin, filter_dict, shots, (gifData)=>{
        PcConfig.callSaveContent(photoData, true, (id_photo)=>{
          if (format.printing) this.printAll(id_photo);
          PcConfig.callSaveGif(gifData, id_photo, onSuccess);
        });
      });
    }
    else
    {
      PcConfig.callSaveContent(photoData, PcConfig.screen.format.idActive, (id_photo)=>{
        if (format.printing) this.printAll(id_photo);

        if (PcConfig.screen.format.idActive) // ugly hack using file_gif field in local database to save photo_id. 
        /* note: idFrame is a frame cropped by Tiny Face Detector and it is not supposed to be removeBged immediately. 
        it should be done delayed, just like below with additional callSaveGif args
        */
        {
          var canvas = PcTemplates.getZoomCropCanvas(PcCaptureSession.idFrame.canvas, PcConfig.screen.format.idWidth,PcConfig.screen.format.idHeight, false);
          var idData = PcConfig.canvasToData(canvas); // iddata -> jpeg data ready
          PcConfig.callSaveGif(idData, id_photo, onSuccess,
            PcConfig.screen.format.idActive,
            PcConfig.screen.format.idRemoveBgActive,
            PcConfig.screen.format.idBackground != "" ? PcConfig.screen.format.idBackground : undefined); 
          /* Note: 
          callSaveGif^^^ will set phocabine_photo line sync field to NULL (as usual)
          or -1 if removebg is required.
          In the later case, process_removebg.py daemon will set it to NULL when composition is done
          so that synchronization with server can happen
          */
        }

        setTimeout(()=>{if (onSuccess) onSuccess();}, 1000);
      });
    }
    PcDbg.log(CRASHDBG, "exit saveAll");
  }

  printAll(id_photo)
  {
    var skin = PcConfig.getMemory("currentSkin");
    var skinColor = skin.color!="" ? skin.color : "#FFFFFF";

    $.ajax
  	({
  		url: 'http://localhost/scripts/getmediatoprint.php?id=' + id_photo + "&bgcolor=" + skinColor,
  		success: (response) => {
  			var r = $.parseJSON(response);
  			var file = "c:/www/photos/" + r.file;
  			var bgcolor = skinColor.replace("#", "");
  			var prints = PcConfig.getMemory("prints")!="" ? PcConfig.getMemory("prints") : 1;
  			var crop = r.crop;
  			var cmd = "c:/www/printer/autoprint.exe \"" + file + "\" " + bgcolor + " " + prints + " " + crop;
        //console.log(cmd);
        if (PcConfig.useElectron())
        {
          let process = require('electron').remote.require("child_process");
          process.exec(cmd);
        }
  		}
  	});
  }

  changeLanguage(l)
  {
    if (PcConfig.currentLanguage==l) return;

    PcConfig.currentLanguage=l;

    let screenStates = {};
    Object.keys(PcScreenIndex).forEach(index => {screenStates[PcScreenIndex[index]] = false;});
    this.setState({screenActive : screenStates});

    setTimeout(()=>{
      screenStates[this.currentScreenIndex] = true;
      this.setState({screenActive : screenStates})
    }, 100);
  }

  render()
  {
    let globalWidth=0;
    let globalHeight=0;

    if (this.props.mode=="photokiosk")
    {
      globalWidth = 1080 * this.props.renderSize;
      globalHeight = 1920 * this.props.renderSize;
    }
    else if (this.props.mode=="photobooth")
    {
      if (this.props.device=="surface")
      {
        globalWidth = 1920 * this.props.renderSize;
        globalHeight = globalWidth * 2 / 3;
      }
      else
      {
        globalWidth = 1920 * this.props.renderSize;
        globalHeight = 1080 * this.props.renderSize;
      }
    }
    else if (this.props.mode=="cyclomaton")
    {
      globalWidth = 1920 * this.props.renderSize;
      globalHeight = 1080 * this.props.renderSize;
    }
    else
    {
      globalWidth = 1080 * this.props.renderSize;
      globalHeight = 1920 * this.props.renderSize;
    }

    return e("div",
      {
        className: this.props.view=="preview" ? "pcscenariopreview" :  "pcscenario",
        style: {
          width:globalWidth,
          height:globalHeight,
          backgroundColor:PcConfig.scenario.backgroundColor,
          border: this.props.view=="preview" ? parseInt(4*this.props.renderSize)+"px solid #888888" : "",
        },
        onClick:()=>{this.resetSleepTimer("onClick")}, // the sleepTimer is handlded at PcScenario div level, above any PcScreen
        onMouseMove:()=>this.resetSleepTimer("onMouseMove"),
        onTouchEnd:()=>this.resetSleepTimer("onTouchEnd"),
      },
      e(PcMedia,
        {
          width:globalWidth,
          height:globalHeight,
          content:PcConfig.scenario.background
        }
      ),
      e(PcMedia,
        {
          width:globalWidth,
          height:globalHeight,
          content:this.state.screenBackground
        }
      ),
      e(PcScreenOutOfOrder,
        {
          active:this.state.screenActive[PcScreenIndex.OutOfOrder],
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.OutOfOrder),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.OutOfOrder),
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcScreenIntro,
        {
          active:this.state.screenActive[PcScreenIndex.Intro],
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.Intro),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenFormat,
        {
          active:this.state.screenActive[PcScreenIndex.Format],
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.Format),
          goNextScreen:(e)=>{ /* if we are in pay mode, PcScreenFormat.props.goNextScreen() goes to PaymentAsk, else directly goes to Format */
            // do we need to pay?
            var payment_active = PcConfig.scenario.payment.active;
            // do we already have a credit? 
            // FIXME: check if cabin_status_xml.payment_credit works
            var payment_credit = parseInt(PcConfig.coalesce(PcConfig.cabin_status_xml.payment_credit, "_value"));
            if (! payment_active || payment_credit) {
              this.goNextScreen(e, PcScreenIndex.Format);
            } else { 
              this.goScreen(PcScreenIndex.PaymentAsk);
            }
          },
          width:globalWidth,
          height:globalHeight
        }
      ),

      e(PcScreenPaymentAsk,
        {
          active:this.state.screenActive[PcScreenIndex.PaymentAsk],
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.PaymentAsk),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.PaymentAsk),
          width:globalWidth,
          height:globalHeight,
        }
      ),

      e(PcScreenPaymentConfirm,
        {
          active:this.state.screenActive[PcScreenIndex.PaymentConfirm],
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.PaymentConfirm),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.PaymentConfirm),
          width:globalWidth,
          height:globalHeight,
        }
      ),

      e(PcScreenStart,
        {
          active:this.state.screenActive[PcScreenIndex.Start],
          has_paymode_bar_timeout:true,
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.Start),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.Start),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenCapture,
        {
          active:this.state.screenActive[PcScreenIndex.Capture],
          startRecording:()=>this.screenCamera.current.startRecording(),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.Capture),
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcScreenLoading,
        {
          active:this.state.screenActive[PcScreenIndex.Loading],
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.Loading),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenTemplateAndFilter,
        {
          active:this.state.screenActive[PcScreenIndex.TemplateAndFilter],
          has_paymode_bar_timeout:true,
          shootAttemptCount:this.state.shootAttemptCount,
          goPrevScreen:(e)=>
            { /* if we are in pay mode, PcScreenTemplateAndFilter.props.goPrevScreen() goes to PcScreenStart, else directly goes to Format */
            // do we need to pay?
            var payment_active = PcConfig.scenario.payment.active;
            if (! payment_active) {
              this.goPrevScreen(e, PcScreenIndex.TemplateAndFilter);
            } else { 
              this.goScreen(PcScreenIndex.Start);
            }
          },
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.TemplateAndFilter),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenInfos,
        {
          active:this.state.screenActive[PcScreenIndex.Infos],
          has_paymode_bar_timeout:true,
          keepState:this.state.formKeepState,
          ref:this.screenInfos,
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.Infos),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.Infos),
          width:globalWidth,
          height:globalHeight
        }
      ),
      e(PcScreenEnd,
        {
          active:this.state.screenActive[PcScreenIndex.End],
          goPrevScreen:(e)=>this.goPrevScreen(e, PcScreenIndex.End),
          goNextScreen:(e)=>this.goNextScreen(e, PcScreenIndex.End),
          width:globalWidth,
          height:globalHeight,
          endLoaded:this.state.endLoaded,
        }
      ),
      e(PcLanguages,
        {
          visible:this.state.langButtonVisible,
          changeLanguage:(l)=>{this.changeLanguage(l)},
          x:0,
          y:0,
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcConnection,
        {
          visible:false,
          x:0,
          y:0,
          width:globalWidth,
          height:globalHeight,
        }
      ),
      e(PcScreenMainLoading,
        {
          display:this.state.displayMainLoader,
          width:globalWidth,
          height:globalHeight,
          content:PcConfig.scenario.background
        }
      )
    );
  }
}
