const MODEL_URL = 'js/lib/face-api/weights';
const LEGACYDBG = true;
const TIMINGDBG = false;


faceapi.env.monkeyPatch({
    Canvas: HTMLCanvasElement,
    createCanvasElement: () => document.createElement('canvas')
});

class PcFraming
{
  constructor()
  {
    this.faceRatio = 2.8;
    this.centerRatio = 0.33;
    this.centerTranslation = 0;
    this.frameSize = {width:3000, height:3000};

    this.finalRatio = 1;
    this.resultWidth = 3000;
    this.resultHeight = 3000;

    this.originalImage = null;

    this.originalCanvasSampled = null;
    this.finalCanvas = null;
    this.sampleRatio = 8;

    this.faceWidth = 0;
    this.faceHeight = 0;
    this.faceRect = null;
    this.mouthX = 0;
    this.mouthY = 0;

    this.fileSaved = false;
    this.outputFile = "";

    this.onInitialized = null;
    this.onRunned = null;

    this.alert = false;
  }

  async initialize(onInitialized)
  {
    faceapi.tf.setBackend('cpu');

    //await faceapi.loadSsdMobilenetv1Model(MODEL_URL);
    await faceapi.loadTinyFaceDetectorModel(MODEL_URL);
    await faceapi.loadFaceLandmarkModel(MODEL_URL);

    this.onInitialized = onInitialized;
    if (this.onInitialized) this.onInitialized();
  }

  async apply(predef_landmarks, canvas, faceRatio, centerRatio, frameSize, alert)
  {
    let t0 = performance.now();
    let canvasRatio = canvas.width/canvas.height;
    let frameRatio = frameSize.width/frameSize.height;
    /* if recognitionThreshold and inputSize parameters are not provided from the XML, use those (416/40%).
    they are currently close to the TFD defaults to be on the safe side.
    Ideally we should never use them and always have the related XML fields populated
    */
    const MY_DEFAULT_RECOGNITION_THRESHOLD = 50; //TFD default
    const MY_DEFAULT_RESIZE_WIDTH = 416; // TFD default
    

    PcDbg.log(LEGACYDBG, "PCFraming:: FRAMING", "au mieux", !alert);
    PcDbg.log(LEGACYDBG, " predef_landmarks", predef_landmarks? "provided": "null");
    PcDbg.log(LEGACYDBG, " canvas w" , canvas.width, "h",  canvas.height);
    PcDbg.log(FRAMINGDBG, "  faceRatio", faceRatio);
    PcDbg.log(FRAMINGDBG, "  centerRatio", JSON.stringify(centerRatio));
    PcDbg.log(FRAMINGDBG, "  frameSize", JSON.stringify(frameSize));

    this.faceRatio = 100/faceRatio;
    this.centerRatio = centerRatio/100;

    this.alert = alert;

    this.resultWidth = frameSize.width;
    this.resultHeight = frameSize.height;
    this.finalRatio = frameSize.width/frameSize.height;
    this.preDownscale =  PcConfig.screen.format.idPreDownscale;
    PcDbg.log(FRAMINGDBG, "  finalRatio", this.finalRatio);
    PcDbg.log(LEGACYDBG, " slow mode:", PcConfig.screen.format.idModeSlow);
    PcDbg.log(LEGACYDBG, " inputSize:", PcConfig.screen.format.idResizedWidth);
    PcDbg.log(LEGACYDBG, " recognitionThreshold:", PcConfig.screen.format.idRecognitionThreshold);
    PcDbg.log(LEGACYDBG, " pre-downscale: ", this.preDownscale);
    PcDbg.log(LEGACYDBG, " show landmarks: ", PcConfig.screen.format.idShowLandmarks);
    /* 
    means legacy behavior: prescaling image at 600px width before giving to TFD
    and sending bogus params to TFD = letting TFD use its own defaults = inputSize 416 and recognitionThreshold 50%
    which means that TFD does another (internal) conversion from 600px width to 416px width
    */
    if (PcConfig.screen.format.idModeSlow) {
      this.targetWidth = 600; // intermediate value, both used to compute sampleRatio and possibly provided to detectFace algorithm parameters (ex: TFD:inputSize)
    } else {

      PcDbg.log(FRAMINGDBG, "  initially: recognitionThreshold: " , PcConfig.screen.format.idRecognitionThreshold, "resizedWidth: " , PcConfig.screen.format.idResizedWidth);
      this.recognitionThreshold = PcConfig.screen.format.idRecognitionThreshold > 0 ? PcConfig.screen.format.idRecognitionThreshold : MY_DEFAULT_RECOGNITION_THRESHOLD;
      this.targetWidth = PcConfig.screen.format.idResizedWidth > 0 ? PcConfig.screen.format.idResizedWidth : MY_DEFAULT_RESIZE_WIDTH;
      this .targetWidth = Math.round(this.targetWidth/32) * 32; //rounding to 21
      PcDbg.log(FRAMINGDBG, "  ->finally: recognitionThreshold" , this.recognitionThreshold, "targetWidth" , this.targetWidth);
    }

    if (this.preDownscale) { 
      this.sampleRatio = canvas.width/this.targetWidth;
    } else {
       this.sampleRatio = 1; 
       /*
       createOriginalCanvasSampled() and detectFace()  rely on this sampleRatio
       to recreate face box with respect to the original picture size (aka canvas aka originalImage)
       */ 
    }
      
    PcDbg.log(LEGACYDBG, " scaling ratio: " , this.sampleRatio);  
    this.createOriginalCanvasSampled(canvas);
    //PcDbg.log(FRAMINGDBG, "sampled");
    this.originalImage = canvas;

    var landmarks = await this.detectFace(predef_landmarks);
    this.computeResult(landmarks);

    this.originalCanvasSampled = null;
    let t1 = performance.now();
    PcDbg.log(TIMINGDBG||LEGACYDBG," FINALLY: apply: took " + (t1 - t0) + " milliseconds.");
    
    return [this.finalCanvas, landmarks];
  }

  createOriginalCanvasSampled(canvas)
  {
    if (this.preDownscale) {
      var t0 = performance.now();
      this.originalCanvasSampled = document.createElement("canvas");
      this.originalCanvasSampled.width = canvas.width / this.sampleRatio;
      this.originalCanvasSampled.height = canvas.height / this.sampleRatio;
      var ctx = this.originalCanvasSampled.getContext('2d');
      ctx.drawImage(canvas, 0, 0, this.originalCanvasSampled.width, this.originalCanvasSampled.height);
      var t1 = performance.now();
      PcDbg.log(TIMINGDBG,"  CreateOriginalCanvasSampled: with canvas ", canvas.width, "x", canvas.height, "and sampleRatio", this.sampleRatio, "scaling took " + (t1 - t0) + " milliseconds.");
    } else { // no prescaling. saves 30 to 300 ms on a ryzen 7 3700x in powersave mode
      this.originalCanvasSampled = canvas;
    }
  }

  async detectFace(predef_landmarks=null)
  {
      //const options = new faceapi.SsdMobilenetv1Options({ minConfidence: 0.1 }); //mobilenet specific
      //const options = new faceapi.TinyFaceDetectorOptions({ scoreThreshold : 0.7, inputSize : 160 }); //tfd specific
    let t0 = performance.now();
    let results = null;
    

    if (predef_landmarks == null ) {
      var options; 
      if (PcConfig.screen.format.idModeSlow) {
        PcDbg.log(FRAMINGDBG, "  using slow/legacy options");
        options = new faceapi.TinyFaceDetectorOptions({ minConfidence: 0.1 }); // bogus Mobilenet params! thus TFD reverts to its defaults: inputSize = 416, recognitionThreshold = 50% 
      } else {
        PcDbg.log(FRAMINGDBG, "  using actual options");
        options = new faceapi.TinyFaceDetectorOptions({ inputSize: this.targetWidth, scoreThreshold: this.recognitionThreshold / 100});
      }

      var my_img = this.preDownscale ? this.originalCanvasSampled : this.originalImage;
      PcDbg.log(LEGACYDBG, " detecting face from image of size", my_img.width, my_img.height);
      results = await faceapi.detectSingleFace(my_img, options).withFaceLandmarks(); // can be null if detectSingleFace failed.

      PcDbg.log(FRAMINGDBG, "with resulting landmarks", JSON.stringify(results));
    } else {
      PcDbg.log(FRAMINGDBG, "  accelerating by using predef_landmarks", JSON.stringify(predef_landmarks));
      results = predef_landmarks; // can be valid or -1.
    }
    if (results == -1 /* with predef_landmarks coming from previous recognition failure  */ 
    || results == null
     ) /* with no predef_landmarks and detectSingleFace genuinely didn't detect any face */
    {
      this.faceWidth = 0;
      this.mouthX = 0;
      this.mouthY = 0;
      this.faceRect = {width:this.originalImage.width, height:this.originalImage.height, x:0, y:0};
      console.warn("No face detected");
      PcConfig.framingAlert.push("face");
      results = -1; 
      /*
      we transform failed detectSingleFace results from null to -1 
      so that if we call detectFace() again with failed predef_landmarks = -1, we fail here quickly instead of slowly recomputing the landmarks
      ( as the predef_landmarks would otherwhise been stored null from the failed previous call!)
      and... refail, which was obvious from the beginning. 
      let's refail quickly instead!
      */
      return results ;
    }
    else {
      PcDbg.log(LEGACYDBG, " face detected");
      PcDbg.log(FRAMINGDBG," with results", JSON.stringify(results));
    }

    const nose = results.landmarks.getNose();
    const mouth = results.landmarks.getMouth();
    const leftEye = results.landmarks.getLeftEye();
    const rightEye = results.landmarks.getRightEye();
    const leftEyeBrow = results.landmarks.getLeftEyeBrow();
    const rightEyeBrow = results.landmarks.getRightEyeBrow();
    const jawOutline = results.landmarks.getJawOutline();

    var minMouthX = Infinity;
    var minMouthY = Infinity;
    var maxMouthX = 0;
    var maxMouthY = 0;

    var zone = nose.concat(mouth, leftEye, rightEye, leftEyeBrow, rightEyeBrow, jawOutline);

    zone.forEach((e)=> {
      if (e.x<minMouthX) minMouthX = e.x;
      if (e.x>maxMouthX) maxMouthX = e.x;
      if (e.y<minMouthY) minMouthY = e.y;
      if (e.y>maxMouthY) maxMouthY = e.y;
    })

    this.faceRect = {};
    this.faceRect.width = (maxMouthX-minMouthX) * this.sampleRatio;
    this.faceRect.height = (maxMouthY-minMouthY) * this.sampleRatio;
    this.faceRect.x = minMouthX * this.sampleRatio;
    this.faceRect.y = minMouthY * this.sampleRatio;

    this.mouthX = this.faceRect.x + this.faceRect.width/2;
    this.mouthY = this.faceRect.y;

    this.faceWidth = this.faceRect.width;
    this.faceHeight = this.faceRect.height;

    PcDbg.log(LEGACYDBG, " faceRect", JSON.stringify(this.faceRect));
    let t1 = performance.now();
    PcDbg.log(TIMINGDBG,"  detectFace: took " + (t1 - t0) + " milliseconds.");
    return results;
  }

  computeResult(landmarks)
  {
    let t0 = performance.now();
    
    let getFinalX = (w) => {
      return this.mouthX - 0.5*w;
    };

    let getFinalY = (h) => {
      return this.mouthY - this.centerRatio*h;
    };

    if (this.faceWidth==0)
    {
      this.finalCanvas = this.originalImage;
      return;
    }

    var finalWidth = this.faceWidth * this.faceRatio;
    var finalHeight = finalWidth / this.finalRatio;

    var finalX = getFinalX(finalWidth);
    var finalY = getFinalY(finalHeight);

    PcDbg.log(FRAMINGDBG, "  computeResult: with faceWidth", this.faceWidth, "faceRatio", this.faceRatio, "finalRatio", this.finalRatio);
    PcDbg.log(FRAMINGDBG, "  computeResult: finalX",finalX, "finalY", finalY, "finalWidth", finalWidth, "finalHeight", finalHeight); 
    var decalX = 0;
    var decalY = 0;

    /*
    if (finalWidth==0)
    {
      finalHeight = this.originalImage.height;
      finalWidth = finalHeight / this.finalRatio;
      finalX = (this.originalImage.width - finalWidth)/2;
      finalY = (this.originalImage.height - finalHeight)/2;
      this.mouthX = this.originalImage.width/2;
      this.mouthY = this.originalImage.height/2;
    }

    if (finalHeight>this.originalImage.height)
    {
      let dif = finalHeight - this.originalImage.height;
      finalHeight = this.originalImage.height;
      finalWidth = finalHeight * this.finalRatio;
      finalY = 0;
      finalX = getFinalX(finalWidth);
      console.log("error height");
      //if (this.alert && dif>0.15*finalHeight) PcConfig.framingAlert.push("size");
      if (this.alert) PcConfig.framingAlert.push("size");
    }

    if (finalWidth>this.originalImage.width)
    {
      let dif = finalWidth - this.originalImage.width;
      finalWidth = this.originalImage.width;
      finalHeight = finalWidth / this.finalRatio;
      finalX = 0;
      finalY = getFinalY(finalHeight);
      console.log("error width");
      //if (this.alert && dif>0.15*finalWidth) PcConfig.framingAlert.push("size");
      if (this.alert) PcConfig.framingAlert.push("size");
    }

    if ((finalY + finalHeight) > this.originalImage.height)
    {
      let dif = ((finalY + finalHeight) - this.originalImage.height);
      finalY = finalY - dif;
      //decalY = Math.max(-(this.originalImage.height-this.fheight)/2, -dif);
      console.log("error down");
      if (this.alert) PcConfig.framingAlert.push("down");
    }

    if (finalY < 0)
    {
      let dif = -finalY;
      finalY = finalY + dif;
      //decalY = Math.min((this.originalImage.height-this.fheight)/2, dif);
      console.log("error up");
      if (this.alert) PcConfig.framingAlert.push("up");
    }

    if ((this.mouthX + 0.5*finalWidth) > this.originalImage.width)
    {
      let dif = (this.mouthX + 0.5*finalWidth) - this.originalImage.width;
      finalX = finalX - dif;
      //decalX = Math.max(-(this.originalImage.width-this.fwidth)/2, -dif);
      console.log("error right");
      //if (this.alert && dif>0.05*finalWidth) PcConfig.framingAlert.push("right");
      if (this.alert) PcConfig.framingAlert.push("right");
    }

    if ((this.mouthX - 0.5*finalWidth) < 0)
    {
      let dif = 0.5*finalWidth - this.mouthX;
      finalX = finalX + dif;
      //decalX = Math.min((this.originalImage.width-this.fwidth)/2, dif);
      console.log("error left");
      //if (this.alert && dif>0.05*finalWidth) PcConfig.framingAlert.push("left");
      if (this.alert) PcConfig.framingAlert.push("left");
    }
    */


    if (finalWidth>this.originalImage.width)
    {
      finalWidth = this.originalImage.width;
      finalHeight = finalWidth / this.finalRatio;
      finalX = 0;
      finalY = getFinalY(finalHeight);
      PcDbg.log(true, "error size (width)");
      PcConfig.framingAlert.push("size");
    }

    if (finalHeight>this.originalImage.height)
    {
      finalHeight = this.originalImage.height;
      finalWidth = finalHeight * this.finalRatio;
      finalX = getFinalX(finalWidth);
      finalY = 0;
      PcDbg.log(true, "error size (height)");
      PcConfig.framingAlert.push("size");
    }

    if (finalX<0)
    {
      finalX=0;
      PcDbg.log(true, "error left");
      if (this.alert) PcConfig.framingAlert.push("left");
    }

    if ((finalX+finalWidth)>this.originalImage.width)
    {
      finalX=this.originalImage.width-finalWidth;
      PcDbg.log(true, "error right");
      if (this.alert) PcConfig.framingAlert.push("right");
    }

    if (finalY<0)
    {
      finalY=0;
      PcDbg.log(true, "error up");
      if (this.alert) PcConfig.framingAlert.push("up");
    }

    if ((finalY+finalHeight)>this.originalImage.height)
    {
      finalY=this.originalImage.height-finalHeight;
      PcDbg.log(true, "error down");
      if (this.alert) PcConfig.framingAlert.push("down");
    }

    PcDbg.log(FRAMINGDBG, "  computeResult: after: ",finalX, finalY, finalWidth, finalHeight);

    var originalCanvas = document.createElement("canvas");
    originalCanvas.width = this.originalImage.width;
    originalCanvas.height = this.originalImage.height;
    var ctx = originalCanvas.getContext('2d');
    ctx.drawImage(this.originalImage, 0, 0, originalCanvas.width, originalCanvas.height);
    if (PcConfig.screen.format.idShowLandmarks) {
      if (landmarks) {
        //PcDbg.log(true, "landmarks are", JSON.stringify(landmarks));

        const detectionsWithLandmarksForSize = faceapi.resizeResults(landmarks, { width: originalCanvas.width, height: originalCanvas.height })
        const drawOptions = {
          drawLines: true,
          drawPoints: true,
          lineWidth: 6,
          pointSize: 6,
          /*
          lineColor: "rgba(255,255,255,1)",
          pointColor: "rgba(0,0,0,1)"
          */
        };
        
        const drawFaceLandMarks = new faceapi.draw.DrawFaceLandmarks(detectionsWithLandmarksForSize.landmarks /* UNDOCUMENTED YET VALID !*/, drawOptions); // undocumented, only visible in code at face-api.js::drawFaceLandmarks():2757, it possibly looks after f.landmarks. This is what we need 
        drawFaceLandMarks.draw(originalCanvas);
        //faceapi.draw.drawFaceLandmarks(originalCanvas, detectionsWithLandmarksForSize, drawOptions);
        PcDbg.log(LEGACYDBG, "  computeResult: please check the landmarks on screen");
      } else {
        PcDbg.log(LEGACYDBG, "  computeResult: no landmarks to draw");
      }
    }
    //facerect
    /*
    ctx.beginPath();
    ctx.lineWidth = "2";
    ctx.strokeStyle = "blue";
    ctx.rect(this.faceRect.x, this.faceRect.y, this.faceRect.width, this.faceRect.height);
    ctx.stroke();

    //center
    ctx.beginPath();
    ctx.lineWidth = "4";
    ctx.strokeStyle = "green";
    ctx.rect(this.mouthX-2, this.mouthY-2, 4, 4);
    ctx.stroke();
    */

    this.finalCanvas = document.createElement("canvas");
    this.finalCanvas.width = this.resultWidth;
    this.finalCanvas.height = this.resultHeight;

    this.finalCanvas.getContext('2d').fillStyle = "black";
    this.finalCanvas.getContext('2d').fillRect(0, 0, this.finalCanvas.width, this.finalCanvas.height);

    //this.finalCanvas.getContext('2d').drawImage(originalCanvas, finalX, finalY, finalWidth, finalHeight, 0, 0, this.resultWidth, this.resultHeight);

    let scale = this.resultWidth/finalWidth;

    if (scale<1)
    {
      PcDbg.log(LEGACYDBG, " computeResult: applying downscale with ratio", scale);
      this.finalCanvas.width = finalWidth;
      this.finalCanvas.height = finalHeight;
      this.finalCanvas.getContext('2d').drawImage(originalCanvas, finalX, finalY, finalWidth, finalHeight, 0, 0, finalWidth, finalHeight);
      this.finalCanvas = this.downScaleCanvas(this.finalCanvas, scale); // takes some time! 300ms on Ryzen 7 3700x VM

      // resolve left pixel column error
      this.finalCanvas.getContext('2d').drawImage(this.finalCanvas, 1, 0, finalWidth-1, finalHeight, 0, 0, finalWidth, finalHeight);
    }
    else {
      this.finalCanvas.getContext('2d').drawImage(originalCanvas, finalX, finalY, finalWidth, finalHeight, 0, 0, this.resultWidth, this.resultHeight);
    }
    let t1 = performance.now();
    PcDbg.log(TIMINGDBG,"  computeResult: took " + (t1 - t0) + " milliseconds.");
    
  }


  downScaleCanvas(cv, scale)
  {
      if (!(scale < 1) || !(scale > 0)) throw ('scale must be a positive number <1 ');
      var sqScale = scale * scale; // square scale = area of source pixel within target
      var sw = cv.width; // source image width
      var sh = cv.height; // source image height
      var tw = Math.floor(sw * scale); // target image width
      var th = Math.floor(sh * scale); // target image height
      var sx = 0, sy = 0, sIndex = 0; // source x,y, index within source array
      var tx = 0, ty = 0, yIndex = 0, tIndex = 0; // target x,y, x,y index within target array
      var tX = 0, tY = 0; // rounded tx, ty
      var w = 0, nw = 0, wx = 0, nwx = 0, wy = 0, nwy = 0; // weight / next weight x / y
      // weight is weight of current source point within target.
      // next weight is weight of current source point within next target's point.
      var crossX = false; // does scaled px cross its current px right border ?
      var crossY = false; // does scaled px cross its current px bottom border ?
      var sBuffer = cv.getContext('2d').
      getImageData(0, 0, sw, sh).data; // source buffer 8 bit rgba
      var tBuffer = new Float32Array(3 * tw * th); // target buffer Float32 rgb
      var sR = 0, sG = 0,  sB = 0; // source's current point r,g,b
      /* untested !
      var sA = 0;  //source alpha  */

      for (sy = 0; sy < sh; sy++) {
          ty = sy * scale; // y src position within target
          tY = 0 | ty;     // rounded : target pixel's y
          yIndex = 3 * tY * tw;  // line index within target array
          crossY = (tY != (0 | ty + scale));
          if (crossY) { // if pixel is crossing botton target pixel
              wy = (tY + 1 - ty); // weight of point within target pixel
              nwy = (ty + scale - tY - 1); // ... within y+1 target pixel
          }
          for (sx = 0; sx < sw; sx++, sIndex += 4) {
              tx = sx * scale; // x src position within target
              tX = 0 |  tx;    // rounded : target pixel's x
              tIndex = yIndex + tX * 3; // target pixel index within target array
              crossX = (tX != (0 | tx + scale));
              if (crossX) { // if pixel is crossing target pixel's right
                  wx = (tX + 1 - tx); // weight of point within target pixel
                  nwx = (tx + scale - tX - 1); // ... within x+1 target pixel
              }
              sR = sBuffer[sIndex    ];   // retrieving r,g,b for curr src px.
              sG = sBuffer[sIndex + 1];
              sB = sBuffer[sIndex + 2];

              /* !! untested : handling alpha !!
                 sA = sBuffer[sIndex + 3];
                 if (!sA) continue;
                 if (sA != 0xFF) {
                     sR = (sR * sA) >> 8;  // or use /256 instead ??
                     sG = (sG * sA) >> 8;
                     sB = (sB * sA) >> 8;
                 }
              */
              if (!crossX && !crossY) { // pixel does not cross
                  // just add components weighted by squared scale.
                  tBuffer[tIndex    ] += sR * sqScale;
                  tBuffer[tIndex + 1] += sG * sqScale;
                  tBuffer[tIndex + 2] += sB * sqScale;
              } else if (crossX && !crossY) { // cross on X only
                  w = wx * scale;
                  // add weighted component for current px
                  tBuffer[tIndex    ] += sR * w;
                  tBuffer[tIndex + 1] += sG * w;
                  tBuffer[tIndex + 2] += sB * w;
                  // add weighted component for next (tX+1) px
                  nw = nwx * scale
                  tBuffer[tIndex + 3] += sR * nw;
                  tBuffer[tIndex + 4] += sG * nw;
                  tBuffer[tIndex + 5] += sB * nw;
              } else if (crossY && !crossX) { // cross on Y only
                  w = wy * scale;
                  // add weighted component for current px
                  tBuffer[tIndex    ] += sR * w;
                  tBuffer[tIndex + 1] += sG * w;
                  tBuffer[tIndex + 2] += sB * w;
                  // add weighted component for next (tY+1) px
                  nw = nwy * scale
                  tBuffer[tIndex + 3 * tw    ] += sR * nw;
                  tBuffer[tIndex + 3 * tw + 1] += sG * nw;
                  tBuffer[tIndex + 3 * tw + 2] += sB * nw;
              } else { // crosses both x and y : four target points involved
                  // add weighted component for current px
                  w = wx * wy;
                  tBuffer[tIndex    ] += sR * w;
                  tBuffer[tIndex + 1] += sG * w;
                  tBuffer[tIndex + 2] += sB * w;
                  // for tX + 1; tY px
                  nw = nwx * wy;
                  tBuffer[tIndex + 3] += sR * nw;
                  tBuffer[tIndex + 4] += sG * nw;
                  tBuffer[tIndex + 5] += sB * nw;
                  // for tX ; tY + 1 px
                  nw = wx * nwy;
                  tBuffer[tIndex + 3 * tw    ] += sR * nw;
                  tBuffer[tIndex + 3 * tw + 1] += sG * nw;
                  tBuffer[tIndex + 3 * tw + 2] += sB * nw;
                  // for tX + 1 ; tY +1 px
                  nw = nwx * nwy;
                  tBuffer[tIndex + 3 * tw + 3] += sR * nw;
                  tBuffer[tIndex + 3 * tw + 4] += sG * nw;
                  tBuffer[tIndex + 3 * tw + 5] += sB * nw;
              }
          } // end for sx
      } // end for sy

      // create result canvas
      var resCV = document.createElement('canvas');
      resCV.width = tw;
      resCV.height = th;
      var resCtx = resCV.getContext('2d');
      var imgRes = resCtx.getImageData(0, 0, tw, th);
      var tByteBuffer = imgRes.data;
      // convert float32 array into a UInt8Clamped Array
      var pxIndex = 0; //
      for (sIndex = 0, tIndex = 0; pxIndex < tw * th; sIndex += 3, tIndex += 4, pxIndex++) {
          tByteBuffer[tIndex] = Math.ceil(tBuffer[sIndex]);
          tByteBuffer[tIndex + 1] = Math.ceil(tBuffer[sIndex + 1]);
          tByteBuffer[tIndex + 2] = Math.ceil(tBuffer[sIndex + 2]);
          tByteBuffer[tIndex + 3] = 255;
      }
      // writing result to canvas.
      resCtx.putImageData(imgRes, 0, 0);
      return resCV;
  }

}
