class PcScreenLoading extends PcScreen
{
  constructor(props)
  {
    super(props);
  }

  getNumShoots()
  {
    var currentFormat = PcConfig.getMemory("currentFormat");
    var currentPoses = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout).poses;

    if (currentFormat.animation=="none" || currentFormat.animation=="boomerang")
    {
      return currentPoses.length;
    }
    else
    {
      return 4;
    }
  }
  
  async testShoots() // wait for shoots to be ready before filtering them
  {
    PcDbg.log(CRASHDBG, "entering testShoots");
    PcConfig.setMemory("shootsRemoveBgResult", false);  // consider that removebg has not been called 

    let shootsToWait = this.getNumShoots();

    // New: makeShootsFromWebCam and makeShootsFromVideo are now async/wait
    if (PcConfig.getMemory("currentFormat").animation == "boomerang")
    {
      setTimeout(async ()=> await PcCameraControl.transferVideoRecording(()=> { 
        // ERROR FIXME: setTimeout doesn't care about the async/await promise settlement! it works by luck
          if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==true)
          {
            this.makeShootsFromWebcam(); // eventually calls makeShootsfilters
          } else {
            this.makeShootsFromVideo(); // eventually calls makeShootFilters
          }
        }), 2000);
    }
    else
    {

      if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==false)
      {
        // honest code related to race conditions, like framing not working because the frame onImageLoaded has not been called yet 
        await Promise.all ([PcConfig.promiseSleep(200)]);  // verified needed to parallelize execution, weird
        PcDbg.log(CRASHDBG, "DEFAULT before makeShootsFilters");
        await this.makeShootsFilters();  // verified needed so testShoots doesn't end too soon
        PcDbg.log(CRASHDBG, "DEFAULT after makeShootsFilters");
        /*
          // what was before
          setTimeout(async ()=> await this.makeShootsFilters(), 500); // WARNING FIXME: setTimeout doesn't care about the promise settlement! it works by luck
        */
        return;
      }

      if (PcCaptureSession.shots.length<shootsToWait)
      {
        PcDbg.log(CRASHDBG, "testShoot: recursing because not enum shots");
        //await Promise.all ([this.testShoots(), PcConfig.promiseSleep(100)]); // DO NOT DO THIS, IT CREATES A STACK LOOP
        setTimeout(()=>this.testShoots(), 100);
      }
      else {
        // honest code related to possible race conditions, like framing not working because the frame onImageLoaded has not been called yet 
         PcDbg.log(CRASHDBG, "testShoot: BEFORE AWAIT TIMEOUT PROMISE");
         await Promise.all ([PcConfig.promiseSleep(200)]); // verified needed to parallelize execution, weird
         PcDbg.log(CRASHDBG, "testShoot: NOW BEFORE makeShootsFilters");
         await this.makeShootsFilters(); // verified needed so testShoots doesn't end too soon
         PcDbg.log(CRASHDBG, "testShoot: NOW AFTER makeShootsFilters");
        /* 
          // what was before 
          setTimeout(async ()=>await this.makeShootsFilters(), 500); // WARNING FIXME: setTimeout doesn't care about the async/await promise settlement! it works by luck
        */
      }

    }

  }

  async makeShootsFromWebcam()
  {
    let currentFormat = PcConfig.getMemory("currentFormat");
    let currentPoses = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout).poses;
    let poses = currentPoses.length;

    for (let i = 0; i<poses; i++)
    {
      for (let j=6; j>0; j--) { 
        // makeShootsFromWebCam: addFrame to shot.. FIXME WHY and WHY 6 of them?
        await PcCaptureSession.shot[i].addFrame(PcCaptureSession.shots[i].frames[j].canvas, true); 
        // FIXME: what is the difference btween poses and frames?
      }
    }

    await this.makeShootsFilters();
  }


  async makeShootsFromVideo()
  {
    console.log("makeshootsfromvideo " + Date.now()/1000);

    /*
    PcCameraControl.exitDSLRStrong(()=>{
      console.log("exit DSLR ok");
    });
    */

    function extractFrame(video, canvas, offset) {
      return new Promise((resolve, reject) => {
        video.onseeked = event => {
          let ctx = canvas.getContext('2d');
          ctx.drawImage(video, 0, 0, canvas.width, canvas.height);
          resolve(canvas);
          };
        video.currentTime = offset;
      });
    };

    let videoFile = PcConfig.getMemory('recordedVideoFile');
    let video = document.createElement ("video");
    let canvas = document.createElement('canvas');

    video.onloadedmetadata = async ()=>{
      canvas.width = video.videoWidth/1.5;
      canvas.height = video.videoHeight/1.5;

      let f = null;

      let currentFormat = PcConfig.getMemory("currentFormat");
      let currentPoses = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout).poses;
      let poses = currentPoses.length;

      for (let i = 0; i<poses; i++)
      {
          for (let j=0; j<8; j++)
          {
            let t = PcCameraControl.flashStamp[i] + (j*1.5/7);
            f = await extractFrame(video, canvas, t);
            PcCaptureSession.addFrame(f, true);
          }

          for (let j=6; j>=1; j--) // reverse for boomerang
          {
            let t = PcCameraControl.flashStamp[i] + (j*1.5/7);
            f = await extractFrame(video, canvas, t);
            PcCaptureSession.addFrame(f, true);
          }

          PcCaptureSession.nextShot();
      }

      await this.makeShootsFilters();
    }

    video.preload = true;
    video.src = videoFile;
  }

  
  _continueMakeShootsFilters()
  {
    /*
    for (let i=0; i<PcConfig.screen.templateAndFilter.filters.length; i++)
    {
      PcCaptureSession.filterShots(PcConfig.screen.templateAndFilter.filters[i], this.shootsFilterDict["use_default_shots"]);
    }
    */
    PcDbg.log(CRASHDBG, "entering _continueMakeShootsFilters, will call makeId");
    if(this.shootsFilterDict["allow_winner"] && PcConfig.getMemory("winner")==1) {
      PcConfig.sounds.win.play();
    } 

    this.makeId(this.shootsFilterDict["predefined_frame_landmarks"]);   
    PcDbg.log(CRASHDBG, "leaving _continueMakeShootsFilters");
  }

  async _commonMakeShootsFilters(useDefaultShots, allow_winner, predefined_frame_landmarks)
  { 
    PcDbg.log(CRASHDBG, "entering _commonMakeShootsFilters");
    this.shootsFilterDict = {};
    this.shootsFilterDict["use_default_shots"] = useDefaultShots;
    this.shootsFilterDict["allow_winner"] = allow_winner;
    this.shootsFilterDict["predefined_frame_landmarks"] = predefined_frame_landmarks;

    // 0. check if we want to use removebg for the current format
    let format_index = PcConfig.getMemory("currentFormatIndex");
    let doRemoveBg = PcRemoveBg.isOn(format_index) /* on, opt_on*/ || PcRemoveBg.isOptional(format_index) /* opt_off */;
    let removeBgTimeout = PcRemoveBg.getTimeout(format_index);
    PcDbg.log(REMOVEDBG, "for format", format_index, "removebg timeout is", removeBgTimeout);
    // 1. if removeBg is asked,  launch it
    let removeBgResult;
    if (doRemoveBg) {
      if (removeBgTimeout == undefined) {
        PcDbg.log(REMOVEDBG, "WARNING: removeBgTimeOut is not defined: IT WILL FAIL");
      }
      removeBgResult = await PcCaptureSession.removeBgShots(removeBgTimeout);
    } 

    PcDbg.log(REMOVEDBG, "-> removeBgResult:", removeBgResult);
    PcConfig.setMemory("shootsRemoveBgResult", removeBgResult); //memorize the result of the removebg process over the whole frames->shots chain 

    // 2. continue processing filters now that the removeBg calculation is done
    this._continueMakeShootsFilters(useDefaultShots, allow_winner, predefined_frame_landmarks)
    PcDbg.log(CRASHDBG, "exiting _commonMakeShootsFilters");
        
  }

  async makeShootsFilters() // eventually only called by testShoots (in multiple ways)
  {
    PcDbg.log(CRASHDBG, "entering makeShootsFilters");
    var currentFormat = PcConfig.getMemory("currentFormat");
    let shots_landmarks = []; // for the record. unused later on
    PcConfig.framingAlert.splice(0, PcConfig.framingAlert.length);

      /* make shots filters depending on use of camera or webcam */
    if (PcCameraControl.useDSLR()==false && PcCameraControl.webcamActive==false)
    {
      console.log("make shoots no webcam no dslr");
      await this._commonMakeShootsFilters(/*useDefaultShots=*/true, /*allow_winner=*/false, /*predef_landmarks=*/null);
      //we call makeId() with no predefined landmarks, we cannot skip the time-consuming landmark detection
      
    }
    else {

      if (currentFormat.autocrop)
      {
        let template = PcTemplates.getTemplate(currentFormat.paper, currentFormat.poses, currentFormat.layout);
        let frameSize = {width:template.poses[0].width, height:template.poses[0].height}

        shots_landmarks = PcCaptureSession.applyFraming(async (sl)=>{  // FIXME: check that this async/await works
          // applyFraming applies to all shots []
          //PcDbg.log(FRAMINGDBG,"makeShootsFilters: onApplyFraming CB");
          await this._commonMakeShootsFilters(/*useDefaultShots=*/false, /*allow_winner=*/true, /*predef_landmarks=*/sl[0][0]);

          /* we call makeId WITH predefined shot_landmarks (sl[]) just computed by applyFraming() above ^
          applyFraming() provide shots landmarks ('pose'),
          each shot landmark provides frames landmarks (if each shot is animated for example = consists of several frames)

          as a functional shortcut, we consider that when we do autocrop, we only makeId based on the first frame of the first shot.
          */
          
        }, /*predef_frame_landmarks=*/ null,
        currentFormat.autocropSize,
        currentFormat.autocropPosition,
        frameSize,
        !currentFormat.autocropSoft);
      }
      else
      {
        await this._commonMakeShootsFilters(/*useDefaultShots=*/false, /*allow_winner=*/true, /*predef_landmarks=*/null);

        //we call makeId() with no predefined landmarks, we cannot skip the time-consuming landmark detection
      }
    }
    PcDbg.log(CRASHDBG, "exiting makeShootsFilters");

  }

  makeId(predef_frame_landmarks)
  {
    PcDbg.log(CRASHDBG, "entering makeId (responsible of going next screen)");
    if (PcConfig.screen.format.idActive)
    {
      PcCaptureSession.createIdShot(()=>{
        this.props.goNextScreen();
      }, predef_frame_landmarks, PcConfig.screen.format.idAutocropSize, PcConfig.screen.format.idAutocropPosition, {width:PcConfig.screen.format.idWidth, height:PcConfig.screen.format.idHeight});
    }
    else {
      this.props.goNextScreen();
    }
    PcDbg.log(CRASHDBG, "exiting makeId");
  }

  getWinner(fOnLoad = null)
  {
    PcDbg.log(CRASHDBG, "entering getWinner");
    if (!PcConfig.instantWin.active)
    {
      if (fOnLoad) {
        fOnLoad(); // testShoots, mostly
      }
      PcDbg.log(CRASHDBG, "exiting getWinner, no winner");
      return;
    }

    PcConfig.setMemory("winner", 0);

    $.ajax({
      url: PcConfig.host + 'scripts/get_winner.php?config=' + PcConfig.global.id,
      type: "GET",
      timeout:10000,
      success: (data)=>{
        var json = PcConfig.xmlToJson(PcConfig.XMLParser.parseFromString(data,"text/xml"));
        console.log(json);
        if (json.xml.id_lot._value!=0)
        {
          console.log("WINNER : lot " + json.xml.gain._value + " - id " + json.xml.id_lot._value);
          PcConfig.setMemory("winner", 1);
          PcConfig.setMemory("winnerLot", json.xml.gain._value);
          PcConfig.setMemory("winnerIdLot", json.xml.id_lot._value);
        }
        if (fOnLoad){
          fOnLoad(); // testShoots mostly
        }
      },
      error: (xhr, status, err)=>{
      }
    });

    PcDbg.log(CRASHDBG, "exiting getWinner, WITH winner");
  }
   
  render()
  {
    if (!this.props.active) return null;
    PcDbg.log(CRASHDBG, "entering render");
    this.getWinner(()=>{ //getWinner is the entry point for testShoots, whatever use case, winner or not
      this.testShoots();
    });
    
    PcDbg.log(CRASHDBG, "render: end waiting testShoots, now render animation");
    return e('div',
      {
        className:this.state.className,
        index:this.props.index,
        style:
        {
          display: (this.props.active ? 'block' : 'none'),
          fontFamily:PcConfig.scenario.fontFamily,
          animation:this.props.noAnimation?"":this.state.animationStyle
        },
      },

      e(PcLabel,
        {
          label:PcConfig.screen.loading.title,
          title:true,
          width:this.props.width,
          height:"15%",
          fontSize:PcConfig.scenario.fontSizeBig,
          color: PcConfig.scenario.fontColor,
          showDelay:"0,1s"
        }
      ),

      e(PcAnimationLoading,
        {
          x:(this.props.width-(400*PcConfig.renderSize))/2,
          y:(this.props.height-(400*PcConfig.renderSize))/2,
          color: PcConfig.scenario.fontColor,
          showDelay:"0.2s"
        }
      )

    );
  }
}
