class PcPaymentError { 
    static EPT_STATUS_OK= new PcPaymentError(0);
    static SERIAL_PORT_NOT_FOUND=new PcPaymentError(1);
    static EPT_NOT_FOUND=new PcPaymentError(2);
    static EPT_PAYMENT_WAITING=new PcPaymentError(3);
    static EPT_FAILURE=new PcPaymentError(4);
    static EPT_NOT_REQUIRED=new PcPaymentError(5);
    static EPT_PAYMENT_OK=new PcPaymentError(6);
    static EPT_PAYMENT_FAILURE=new PcPaymentError(7);
    static EPT_PAYMENT_AUTHORISATION_FAILURE=new PcPaymentError(8);
    static EPT_PAYMENT_TIMEOUT=new PcPaymentError(9); // never a card
    static EPT_PAYMENT_DEVICE_CANCEL=new PcPaymentError(10); // bad card type
    static INTERNAL_EXCEPTION=new PcPaymentError(11);
    static EPT_LIBRARY_ERROR=new PcPaymentError(12);
    static EPT_UNHANDLED_ERROR=new PcPaymentError(13);

  constructor(error_code) {
    this.error_code = error_code;
  }
}
class PcPaymentErrorCategories { 
  static NeedsFullRestart = [ // return to Intro screen
    PcPaymentError.EPT_PAYMENT_TIMEOUT
  ];

  static NeedsSimpleRetry = [ 
    PcPaymentError.EPT_PAYMENT_DEVICE_CANCEL,
    PcPaymentError.EPT_PAYMENT_FAILURE,
    PcPaymentError.EPT_PAYMENT_AUTHORISATION_FAILURE,
  ];

  static NeedsGoNextScreen = [ // to go to payment confirm
    PcPaymentError.EPT_PAYMENT_OK,
    PcPaymentError.EPT_STATUS_OK,
  ];

}
class PcScreenPaymentAsk extends PcScreen
{
    constructor(props)
    {
      super(props);
    }

    async queueJob() //
    {
        PcDbg.log(PAYMENTDBG, "entering");
        /* mandatory args */
        var askTimeout = PcConfig.scenario.payment.askTimeout;
        let params = {};
        params["date"] = PcConfig.formatDate(new Date());
        params['id_photocabine_status'] = parseInt(PcConfig.cabin_status_xml.id._value);    
        params["ttl_seconds_record"] = askTimeout; // warning, there is a latency! fine tuning
        params["fee_cents_record"] = PcConfig.scenario.payment.fee * 100;
        params["config_record"] = PcConfig.global.id;

        var post_str =JSON.stringify(params);
        
        PcDbg.log(PAYMENTDBG, "payment params are:", post_str);
        
        var do_loop = true;
        let paymentdbg_data = "nothing yet";
        do{
          try { 
                  paymentdbg_data = await  $.ajax({
                  url: PcConfig.host + 'scripts/queue_job_payment.php?',
                  type: "POST",
                  data: {json_data : post_str},
                  //timeout:(askTimeout+1)*1000
              });
              var return_data = JSON.parse(paymentdbg_data);
              PcDbg.log(PAYMENTDBG, "after queue_job_payment.php, got data", paymentdbg_data, "ie return data", JSON.stringify(return_data));

              let error_code = return_data["error_code"];
              if (error_code == "ok") {
                PcDbg.log(true, ` OK: got ${return_data["error_code_low_level"]}, we must simply go next screen`);
                do_loop = false;
                this.props.goNextScreen();

              } else if (error_code == "try_again") {
                PcDbg.log(true, `TRY_AGAIN ERROR: got ${return_data["error_code_low_level"]}, we must loop payment request`);

              } else if (error_code == "timeout" || error_code == "bad_input" )  {
                PcDbg.log(true, `TIMEOUT OR BAD_INPUT ERROR: got ${return_data["error_code_low_level"]}, we must go back to app intro screen`);

                this.props.goPrevScreen();// something bad happened, go back to intro screen
                do_loop = false;                      
              } else if (error_code == "unknown" || error_code == "fatal") {
                var msg = `FATAL OR  ERROR: ${error_code}, SHOULD NOT HAPPEN -> raise exception to crash app`;
                PcDbg.log(true, msg) ;
                throw new Error(msg);
              }
              // check the kind of error we get  
              /*
              let do_parse = true; // ugly
              if (do_parse) {            
                Object.keys(PcPaymentErrorCategories.NeedsFullRestart).forEach(index =>
                  {
                    if (error_code_int == PcPaymentError[index].err_code) { // timeout
                      PcDbg.log(true, `FATAL ERROR: got ${PcPaymentError[index]}, we must go back to intro screen`);

                        this.goPrevScreen();// something bad happened, go back to intro screen
                        do_loop = false;                      
                        do_parse = false;
                    }
                  }
                );
              }
              if (do_parse) {
                 Object.keys(PcPaymentErrorCategories.NeedsSimpleRetry).forEach(index => //device cancel, payment failure, payment authorisation failure
                  {
                    if (error_code_int == PcPaymentError[index]) {
                      PcDbg.log(true, `NONFATAL ERROR: got ${PcPaymentError[index]}, we must simply loop`);
                       do_parse = false;
                    }
                  }
                );
              }
              if (do_parse) {
                Object.keys(PcPaymentErrorCategories.NeedsGoNextScreen).forEach(index =>
                  {
                    if (error_code_int == PcPaymentError[index]) {
                      PcDbg.log(true, ` OK: got ${PcPaymentError[index]}, we must simply go next screen`);
                      do_loop = false;
                      this.goNextScreen();
                      do_parse = false;
                    }
                  }
                );
              }

              if (do_parse) { // catchall!
                var msg = ` ERROR UNHANLDED ERROR: ${error_code_int}, SHOULD NOT HAPPEN -> raise exception to crash app`;
                PcDbg.log(true, msg) ;
                throw new Error(msg);
              }
            */

          } catch (error) {
              PcDbg.log(true, "ERROR: Ajax request error", error, "payment_data is", paymentdbg_data, "go back to previous screen");
              this.stopNextScreenTimer();
              this.props.goPrevScreen();// something bad happened, go back to intro screen
              do_loop = false;
          }
        } while(do_loop);

        PcDbg.log(PAYMENTDBG, "exiting");
    }
    
  stopNextScreenTimer()
  {
    clearTimeout(this.nextScreenTimer);
  }
  resetNextScreenTimer()
  {
    this.nextScreenTimer = setTimeout(()=>this.onNextScreenTimer(), PcConfig.scenario.payment.askTimeout);
  }
  onNextScreenTimer() 
  {
    PcDbg.log(CRASHDBG, "entering. we should go to the confirm screen now");
    this.props.goNextScreen();
  }

  render()
  {
    if (!this.props.active) return null;
    // if we are here, payment_mode is assumed (see scenario -> PcScreenFormat goNextScreen prop setting)

    // play credit ask sound
    PcConfig.sounds.paymentAsk.play();
    //queue a payment job
    this.queueJob();

    return e('div',
      {
        className:this.state.className,
        index:this.props.index,
        style:
        {
          display: (this.props.active ? 'block' : 'none'),
          fontFamily:PcConfig.scenario.fontFamily,
          animation:this.state.animationStyle
        },
      },
        e('div',
        {
          style:
          {
            position:"absolute",
            width:this.props.width*0.9,
            height:this.props.height*0.2,
            left:this.props.width*0.05,
            top:this.props.height*(PcConfig.screen.paymentAsk.titlePosition=="top" ? 0.05 : (PcConfig.screen.paymentAsk.titlePosition=="bottom" ? 0.75 : 0.4)),
          }
        },
          e(PcLabel,
            {
              label:PcConfig.screen.paymentAsk.title,
              title:true,
              width:this.props.width*0.9,
              height:this.props.height*0.2,
              x:0,
              y:0,
              fontSize:PcConfig.scenario.fontSizeGiant,
              color:PcConfig.screen.intro.titleColor,
              valign:PcConfig.screen.paymentAsk.titlePosition,
              noAnimation:true,
              showDelay:"0.1s"
            }
          )
        )
    );
  }
}
