<?php
include_once 'config.php' ;

const QUEUEDBG = true;

const PYTHON_ERR_CODE_OK = 0;
const PYTHON_ERR_CODE_BUSY = 1;
const PYTHON_ERR_CODE_RBG_TIMEOUT = 2;
const PYTHON_ERR_CODE_RBG_API_FATAL = 3;
const PYTHON_ERR_CODE_INTERNAL_EXCEPTION_FATAL = 4;
//more err codes
const PYTHON_ERR_CODE_INTERNET_DOWN = 5;
const PYTHON_ERR_CODE_RBG_TIMEOUT_FATAL = 6;
const PYTHON_ERR_CODE_INTERNET_DOWN_FATAL = 7;

function leave($data, $error_code, $reason=null)
{
    if ($reason) {
        dbg(true, $reason);
    }
    $data["error_code"] = $error_code;
    print(json_encode($data));
    exit();
}

function check_key_missing($dict, $keys)
{
    $ret = 0;
    dbg(true, "confronting dictionary " . json_encode($dict) . "vs keys ", json_encode($keys));
    foreach ($keys as $k)
    if (!array_key_exists($k, $dict)) {
        dbg(true, "in dictionary " . json_encode($dict) . " key $k is missing");
        $ret = 1;
        break;
    }
    return $ret;
}
$return_data = [];

if(! isset($_POST) ) {
    leave($return_data, ERR_CODE_MISSING_INPUT, "no POST data, quitting");
}
//var_dump($_POST);
dbg(CRASHDBG, "post data is " . json_encode($_POST) . "json_data ---> " . json_encode($_POST["json_data"]));
if (! isset($_POST['json_data'])) {
    leave($return_data, ERR_CODE_BAD_INPUT, "no json_data in POST, quitting");
}
$json_string = $_POST["json_data"];
dbg(CRASHDBG, "json string is " . $json_string);

$json_data = json_decode($json_string, /* as_array= */ true); // warning, $json_data is a stdClass, second argument makes it an arra

dbg(CRASHDBG, "json_data is " . json_encode($json_data));
/*
    //warning: json_decode does not provide a pure array but an stdclass, array_interesect_data either further casting, or json_decode needs second parameter to true.
    but fake_json_data works: 
    $fake_json_data = array("id_photo" => 9999);
    dbg(CRASHDBG, "fake_json_data is " . json_encode($fake_json_data));
*/

$mandatory_table_fields = ["date", "in_filename", "ttl_seconds", "api_key", "out_format"];
if (check_key_missing($json_data, $mandatory_table_fields)) {
    leave($return_data, ERR_CODE_MISSING_INPUT, "some input keys are missing");
}

$optional_table_fields = ["id_photo", "out_filename",  "in_bg_filename", "error_code"]; // optional or out args

$all_table_fields = array_merge($mandatory_table_fields, $optional_table_fields);
$non_table_fields = ["is_id"];
$all_fields = $all_table_fields + $non_table_fields ;
$removebg_table = "photocabine_auto_jobs_removebg";

$insert_data = array_intersect_key($json_data, array_flip($all_table_fields));

dbg(CRASHDBG, "insert_data is " . json_encode($insert_data));


#define output filename
$insert_data["out_filename"] = get_media_filename($insert_data["in_filename"], /*isId=*/$json_data["is_id"] , /*saveFormat=*/$insert_data["out_format"], /*doRemoveBg=*/ true);
$insert_data["out_field_name"] = $json_data["is_id"] ? "file_gif": "file"; // WARNING
dbg(CRASHDBG, "WARNING CHECK OUT_FIELD_NAME CAREFULLY");
dbg(CRASHDBG, "==> insert_data is " . json_encode($insert_data));

dbg(QUEUEDBG, "in $removebg_table, inserting " . json_encode($insert_data));

$insert_id = my_insert($db, $removebg_table, $insert_data);
dbg(QUEUEDBG, "-> got insert_id: ". $insert_id);


$return_data["date"] = $insert_data["date"];
$return_data["in_filename"] = $insert_data["in_filename"];
$return_data["id"] = $insert_id;
$return_data["ttl_seconds"] = $insert_data["ttl_seconds"];
if (isset($insert_data["id_photo"])) {
    $return_data["id_photo"] = $insert_data["id_photo"];
} 


if ($insert_data["ttl_seconds"] <= 0) { #we don't need to synchronously wait on this one
    leave($return_data, ERR_CODE_OK, "delayed job, we don't need to wait actively for it, bye");
}

$JOB_DONE_POLL_DELAY = 0.25;
$sql = "SELECT * from $removebg_table where id = ". $insert_id . " and error_code IN ('";
$sql .= PYTHON_ERR_CODE_OK . "', '";
$sql .= PYTHON_ERR_CODE_RBG_API_FATAL . "', '";
$sql .= PYTHON_ERR_CODE_INTERNAL_EXCEPTION_FATAL . "', '";
$sql .= PYTHON_ERR_CODE_RBG_TIMEOUT_FATAL . "', '";
$sql .= PYTHON_ERR_CODE_INTERNET_DOWN_FATAL . "')";
$then = time();
$processed_in_time = false;
while ((time() - $then) < $insert_data["ttl_seconds"] )
{
    $line = my_select($db, $sql);
    if ($line) {
        dbg(QUEUEDBG, "DONE! waited " . strval(time() - $then)  . " seconds, error_code: ". $line->error_code);
        if ($line->error_code == 0) { 
            $return_data["out_filename"] = $line->out_filename;
            leave($return_data, ERR_CODE_OK, "job ended normally in time");
        } else {
            leave($return_data, ERR_CODE_UNKNOWN, "job ended abnormally with error", $line->error_code);
        }
        $processed_in_time = true;
        break;
    } else {
        dbg(false && QUEUEDBG, "waiting $JOB_DONE_POLL_DELAY seconds");
        usleep($JOB_DONE_POLL_DELAY * ONE_SECOND_IN_USEC);
    }
}
if (! $processed_in_time) {
    leave($return_data, ERR_CODE_TIMEOUT, "job timed out!");
}
leave($return_data, ERR_CODE_UNKNOWN, "UNKNOWN ERROR, should not be there");
?>