<?php
require_once 'synchronize_common.php';
define('SYNCPHOTODBG', false);
define('SYNCVIDEODBG', false);
define('DRY_RUN_FILE_DELETE', false);
define('RETENTIONDBG', false);

if (my_ping() != 0) {
    dbg(true, "skipping");
}

// GLOBALS
$RETENTION_DELAY_SECONDS = 30 * ONE_DAY_IN_SECONDS;
$oConfigXml = simplexml_load_file(getXmlFile('config.xml')) ;
if (! isset($oConfigXml))
{
    dbg(true, "XML file not found, skipping this turn");
    exit;

}
function get_retention_delay_seconds($xml)
{
	global $RETENTION_DELAY_SECONDS;
	$some_vars_set = 0;
	$ret = 0;
	dbg(RETENTIONDBG,"isset: ".  isset($xml->general->configuration->retentionDelayEnable). " " .  isset($xml->general->configuration->retentionDelayDays) . " " . isset($xml->general->configuration->retentionDelayHours) . " " . isset($xml->general->configuration->retentionDelayMinutes)) ;
	dbg(RETENTIONDBG,"retention values: enable: ".  $xml->general->configuration->retentionDelayEnable . " days: " .  $xml->general->configuration->retentionDelayDays . " hours: " . $xml->general->configuration->retentionDelayHours . "  minutes: " . $xml->general->configuration->retentionDelayMinutes) ;

	if (
		isset($xml->general->configuration->retentionDelayEnable)
		&& ($xml->general->configuration->retentionDelayEnable == 1)
	) {
		if (isset($xml->general->configuration->retentionDelayDays)) {
			$some_vars_set = 1;
			$ret += $xml->general->configuration->retentionDelayDays * ONE_DAY_IN_SECONDS;
		}
		if (isset($xml->general->configuration->retentionDelayHours)) {
			$some_vars_set = 1;
			$ret += $xml->general->configuration->retentionDelayHours * ONE_HOUR_IN_SECONDS;
		}
		if (isset($xml->general->configuration->retentionDelayMinutes)) {
			$some_vars_set = 1;
			$ret += $xml->general->configuration->retentionDelayMinutes * ONE_MINUTE_IN_SECONDS;
		}

		dbg(RETENTIONDBG, "total xml retention: ". $ret);
	}
	if ($some_vars_set == 0)  {
		dbg(RETENTIONDBG, "fallback to hardcoded 30 days values:" . $RETENTION_DELAY_SECONDS . " " . ONE_DAY_IN_SECONDS);
		$ret = $RETENTION_DELAY_SECONDS;
	}
	
	dbg(RETENTIONDBG,"final value: ". $ret);
	return $ret;
}

function my_unlink($file)
{
	$ret = NULL;
	if (DRY_RUN_FILE_DELETE)
	{
		drdbg("unlinking $file");
	}
	else
	{
		$ret = unlink($file);
	}
	return $ret;
}



function my_exec($cmd)
{
	$ret = "";
    dbg(EXECDBG, $cmd);
	if (DRY_RUN_EXEC)
	{
        drdbg("exec");
	}
	else
	{
		$ret = exec($cmd);
	}
	return $ret;
}


function getOldDatabaseEntriesSQLRequestBase($xml)
{
    $sql     = 'SELECT * FROM phocabine_photos WHERE sync = 1 AND date < DATE_SUB(NOW(), INTERVAL ' . get_retention_delay_seconds($xml) . ' SECOND)';
    return $sql;
}
function getOldDatabaseEntries($my_db, $xml) // returns an array of dicts with db keys "file", "file_gif", video" and "date" (to help reconstructing path in server)
{
    $old_entries = [];
    $old_entries["entries"] = [];
    $sql =  getOldDatabaseEntriesSQLRequestBase($xml) . ' ORDER BY id_photo' ;
	$lines     = my_selectAll($my_db, $sql) ;
    //dbg(SQLDBG, "lines " . json_encode($lines));
	foreach($lines as $line) {

        $files_to_keep = [];
        $files_to_keep["device"] = $line->device;
        $files_to_keep["config"] = $line->config;
        $files_to_keep["file"] = $line->file;
        $files_to_keep["file_gif"] = $line->file_gif;
        $files_to_keep["video"] = $line->video;
        $files_to_keep["date"] = $line->date; // "date" is to help reconstructing picture/gif/video path in file, file_gif and video fields...

        //dbg(SQLDBG, "TAMERE " . json_encode($files_to_keep));
        $old_entries["entries"][] = $files_to_keep;
    }
    //dbg(SQLDBG, "old_entries (" . sizeof($old_entries["entries"]) . ") are: " . json_encode($old_entries));
    return $old_entries; //OK

}

function purgeOldDatabaseEntries($my_db, $xml) {

    // 1. query old-aged local cabin database lines

    $old_entries = getOldDatabaseEntries($my_db, $xml);
    $old_entries_json = json_encode($old_entries);
    //dbg(SQLDBG, "purgeOldDatabaseEntries: old_entries_json are: " . ($old_entries_json));
    // 2. check if those are on the server
    $ch = curl_init();
    $opts =  array(
        CURLOPT_CONNECTTIMEOUT => 10,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT        => 600,
        CURLOPT_HTTPHEADER, array('Content-Type:application/json')
    );

    $opts[CURLOPT_POSTFIELDS] = $old_entries_json;
    //$opts[CURLOPT_POSTFIELDS] = json_encode(array("customer"=>"popek")); 
    $site_prefix = SSITE; 
    //$site_prefix = "http://localhost";// fixme remove me when ready
    $opts[CURLOPT_URL] = $site_prefix . '/scripts/check_db_entries.php';

    curl_setopt_array($ch, $opts);
 
    curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, false);

    // 3. server shall only respond with entries IT DOES NOT KNOW ABOUT 
    $curl_result = my_curl_exec($ch);
    dbg(EXECDBG, "curl_result: " . $curl_result) ; 
    $json_result = json_decode($curl_result, true);

    // 4. for all entries that are NOT in the server result, proceed with destruction 

    $old_entries_del_count = 0;
    $unknown_entries = $json_result['unknown_entries'];
    if ($unknown_entries === null) {
        dbg(true, "problem with unknown_entries, it is NULL");
        return;
    }
    $sql =  getOldDatabaseEntriesSQLRequestBase($xml) ;
    //print_r($unknown_valid_entries);
    if (sizeof($unknown_entries) == 0){
        dbg(SQLDBG, "no unknown entries in server");
    } else {
        foreach ($unknown_entries as $key => $entry) { 
			$sql .= ' AND  NOT ';
            $sql_tuple = sprintf("( file = '%s' AND file_gif = '%s' AND video = '%s' )",
            $my_db->real_escape_string(basename($entry["file"])),
            $my_db->real_escape_string(basename($entry["file_gif"])),
            $my_db->real_escape_string(basename($entry["video"])));
                
            $sql .= " " .$sql_tuple;
            $old_entries_del_count ++;
            if ($old_entries_del_count < sizeof($unknown_entries)) {
                $sql .= " ";
            }
        }
        $sql .=" ";
    }
    $sql .= " ORDER BY id_photo";
    $lines_to_destroy = my_selectAll($my_db, $sql);


    // 5. we have the local cabin lines we should destroy. 
    
    dbg(SQLDBG, "lines_to_destroy: ". json_encode($lines_to_destroy));
    foreach($lines_to_destroy as $line)
	{
        // 6. destroy the mediums
        if ($line->file)
		{
			my_unlink(getPhotoFile($line->file)) ;
		}
        if ($line->file_gif)
        {
			my_unlink(getPhotoFile($line->file_gif)) ;
        }

		if ($line->video)
		{
			my_unlink(getVideoFile($line->video)) ;
		}
		{
            if (false) {    	drdbg("update photo $line->id_photo sync to 3");
                // 7 . mark the line as destroyed (sync = 3)
    	        my_update($my_db, 'phocabine_photos', array('sync' => 3), 'id_photo = ' . $line-> id_photo) ; // FIXME: PER CUSTOMER REQUEST, THEY SHOULD BE DELETED INSTEAD
		    } else {
                // 7.  destroy the line.
                my_delete($my_db, 'phocabine_photos', 'id_photo = ' . $line-> id_photo);
            }		
        }		
	}


 
}



purgeOldDatabaseEntries($db, $oConfigXml);

//instant win
if ($oConfigXml->photoWin->general->active == 1)
{
    //manual lots won
    $sql     = 'SELECT * FROM photocabine_lots WHERE config = "' . $oConfigXml->id . '" AND won IS NOT NULL AND auto IS NULL' ;
    $lines     = my_selectAll($db, $sql) ;

    $aDates = array() ;
    foreach($lines as $line)
    {
        array_push($aDates, $line->date . '/' . $line->id) ;
    }
    $params['dates'] = implode('|', $aDates) ;

    $ch = curl_init();
    $opts =  array(
        CURLOPT_CONNECTTIMEOUT => 10,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT        => 600
      );
    $opts[CURLOPT_POSTFIELDS] = $params;
    $opts[CURLOPT_URL] = SSITE . 'scripts/upload_lots_date.php?config=' . $oConfigXml->id ;

    curl_setopt_array($ch, $opts);

    //transmit manual lots won
    curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, false);
    $result = my_curl_exec($ch);

    echo "result:".$result ;

    if ($result)
    {
        if ($result != '1')
        {
            $a = explode('|', $result) ;

            //delete local entries not returned by server
            $sql     = 'SELECT * FROM photocabine_lots WHERE config = "' . $oConfigXml->id . '" AND won IS NULL AND auto IS NULL' ;
            $lines     = my_selectAll($db, $sql) ;

            foreach($lines as $line)
            {
                $exist = false;

                foreach($a as $date_info)
                {
                    $a2 = explode('/', $date_info);
                    $date = $a2[0];
                    if ($date==$line->date)
                    {
                        $exist = true;
                        break;
                    }
                }

                if (!$exist)
                {
                    $sql = 'DELETE FROM photocabine_lots WHERE id_lot=' . $line->id_lot ;
                    my_query($db, $sql);
                }
            }

            //create new manual lots gets from online script, if they don't exist
            foreach($a as $date_info)
            {
                $a2 = explode('/', $date_info) ;

                $date = $a2[0] ;

                if ($date!="" && $date!=0 && $date!="0000-00-00 00:00:00")
                {

                    $sql     = 'SELECT * FROM photocabine_lots WHERE config = "' . $oConfigXml->id . '" AND won IS NULL AND auto IS NULL AND date="'.$date.'"' ;
                    $lines     = my_selectAll($db, $sql) ;

                    if (!$lines || count($lines)==0)
                    {
                        $data = array() ;
                        $data['date']     = $date ;
                        $data['gain']     = $a2[1] ;
                        $data['config']    = $oConfigXml->id ;
                        $db->insert('photocabine_lots', $data) ;
                    }
                }
            }
        }
        else
        {
            $sql = 'DELETE FROM photocabine_lots WHERE config = "' . $oConfigXml->id . '" AND won IS NULL AND auto IS NULL';
            my_query($db, $sql);
        }
    }
}


// Sync content online
$sql     = 'SELECT * FROM phocabine_photos WHERE sync IS NULL ORDER BY id_photo' ;
$line     = my_select($db, $sql);
if($line)
{
    my_update($db, 'phocabine_photos', array('sync' => 1), 'id_photo = ' . $line->id_photo) ;

    //echo 'Sync =>' . $line->file . '<br/>' ;
    $ch = curl_init();
    $params = get_object_vars($line) ; // convert db line to hashtable for curl post fields
  
    if (substr($params['email'], 0, 1)=="@") $params['email'] = substr($params['email'],1);
    if (substr($params['form'], 0, 1)=="@") $params['form'] = substr($params['form'],1);

    $params['photo'] = new CurlFile(realpath(getPhotoFile($line->file))) ;

    if ($line->video && getVideoFile($line->video))
    {
        $params['videoo'] = new CurlFile(realpath(getVideoFile($line->video))) ;
    }

    if ($line->file_gif && getPhotoFile($line->file_gif))
    { 
        if (file_exists(getPhotoFile($line->file_gif)))
        { 
            $params['gif'] = new CurlFile(realpath(getPhotoFile($line->file_gif))) ;
        }else {
            dbg(true, "ERR file_not_exists: " . $line->file_gif . "x".getPhotoFile($line->file_gif));
            unset($params['file_gif']) ;
        }
    } else {
        dbg(true, "no file_gif or error on getPhotoFile: " . $line->file_gif);
    }

    $opts =  array(
        CURLOPT_CONNECTTIMEOUT => 10,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT        => 600
      );
    $opts[CURLOPT_POSTFIELDS] = $params;
    $opts[CURLOPT_URL] = SSITE . 'scripts/upload_photo.php' ;

    curl_setopt_array($ch, $opts);
    curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, false);
    $result = my_curl_exec($ch);
    if (EXECDBG) {
        var_dump($opts);
        var_dump($result) ;
    }

    if ($result == '1')
    {
      echo "ok sync";
    }else {
        my_query($db, 'UPDATE phocabine_photos SET sync = NULL WHERE id_photo = ' . $line->id_photo) ;
        echo "ko sync";
		echo ('Erreur Curl : ' . curl_error($ch));
    }

    exit ;
}

// Convert MOV to MP4
$sql     = 'SELECT * FROM phocabine_photos WHERE sync = 0 AND video IS NOT NULL ORDER BY id_photo' ;
$line     = my_select($db, $sql) ;

if($line)
{
    $file = substr($line->video, 0, strpos($line->video, '.')) ;

    $input_video = getVideoFile($file . '.mov');
    $input_silence =  getVideoFile("temp/audio.mp3");

    $hd_video = getVideoFile($file."_hd.mp4");
    $resized_video = getVideoFile("temp/".$file."_video_resized.mp4");

    $overlayed_video =  getVideoFile("temp/".$file."_video_overlayed.mp4");
    $overlayed_video_forconcatfile = "temp/".$file."_video_overlayed.mp4";

    $question_video =  getVideoFile("temp/".$file."_video_question.mp4");
    $question_video_forconcatfile = "temp/".$file."_video_question.mp4";

    $final_video_temp = getVideoFile("temp/".$file.".mp4");
    $final_video = getVideoFile($file . '.mp4');

    $concat_file = getVideoFile($file."_concat.txt");

    $contrast = "1.2"; // -2 to 2 (default 1 : no change)
    $brightness = "0.1"; // -1 to 1 (default 0 : no change)
    $saturation = "1.2"; // 0 to 3 (default 1 : no change)

    $nosound = ($oConfigXml->vstep1->nosound==1) ? "-an" : "";

    //$codec = "-r 25 -b:a 128k -b:v 4000k -g 12 -bf 2 $nosound";
    $codec = "-r 25 -b:a 128k -b:v 4000k -g 12 -bf 2 $nosound -pix_fmt yuv420p";

    $finalvideosize = "1920x1080";
    $fvw = 1920;
    $fvh = 1080;

    $question = $line->video_question ? $line->video_question : 0 ;

    $image_file = $oConfigXml->videoMenu->themes->themes->{"maskPicture".$question} ;
    $image = getRessourceFile($image_file) ;

    //create overlayed video
    if ($image_file != '' && file_exists($image))
    {
        $cmd = FFMPEG . " -i $input_video -i $image -filter_complex overlay -s $finalvideosize $codec -y $overlayed_video";
        my_exec($cmd);
    }
    else
    {
        $cmd = FFMPEG . " -i $input_video -s $finalvideosize $codec -y $overlayed_video";
        my_exec($cmd);
    }

    // Add Image Question
    $image_file = $oConfigXml->videoMenu->themes->themes->{"introPicture".$question} ;
    $image = getRessourceFile($image_file) ;

    if ($image_file != '' && file_exists($image))
    {
        $input_question = getVideoFile("temp/".$file."_question.jpg");
        copy($image, $input_question);

        //create question video
        $cmd = FFMPEG . " -loop 1 -i $input_question -i $input_silence -pix_fmt yuv420p -s $finalvideosize $codec -t 4 -y $question_video";
        my_exec($cmd);

        //concat question video and overlayed video
        $f = fopen($concat_file, "w");
        fwrite($f, "file $question_video_forconcatfile\r\n");
        fwrite($f, "file $overlayed_video_forconcatfile");
        fclose($f);

        $cmd = FFMPEG . " -f concat -i $concat_file -s $finalvideosize $codec -y $final_video" ;
        my_exec($cmd);

        my_unlink($input_question);
        my_unlink($concat_file);

    $thumbtime = 6;
    }
    else
    {
        $thumbtime = 0;
        copy($overlayed_video, $final_video);
    }

    //extract first frame as jpeg
    $thumb = getPhotoFile($line->file);
    $cmd = FFMPEG . " -ss $thumbtime -i $final_video -vframes 1 -s 1280x720 -f image2 -y $thumb";
    my_exec($cmd);

    if(file_exists($resized_video)) my_unlink($resized_video);
    if(file_exists($overlayed_video)) my_unlink($overlayed_video);
    if(file_exists($question_video)) my_unlink($question_video);
    if(file_exists($final_video_temp)) my_unlink($final_video_temp);

    if (file_exists($final_video))
    {
        my_unlink($input_video);
        my_update($db, 'phocabine_photos', array('video' => $file . '.mp4'), 'id_photo = ' . $line->id_photo) ;
    }

    my_query($db, 'UPDATE phocabine_photos SET sync = NULL WHERE id_photo = ' . $line->id_photo) ;

    echo 1 ;

    exit ;
}

// Retrieve Selfie from Internet
if($oConfigXml->params->mode == 3)
{
    $config = $oConfigXml->id ;

    $selfies = @simplexml_load_file(SITE . 'scripts/get_selfies.php?config=' . $config . '&device=' . DEVICE_ID) ;

    if ($selfies && count($selfies->selfie))
    {
        foreach($selfies->selfie as $selfie)
        {
            $context = stream_context_create(array('http' => array('timeout' => 60)));
            $raw = @file_get_contents($selfie->photo, false, $context) ;

            if ($raw)
            {
                $id     = uniqid($config . '_');
                $name     = getPhotoFile($id . '.jpg') ;
                file_put_contents($name, $raw) ;

                $data = array() ;
                $data['config']     = $config ;
                $data['device']     = DEVICE_ID ;
                $data['lang']         = 1 ;
                $data['email']         = $selfie->email ;
                $data['file']         = $id . '.jpg' ;
                $data['notif']         = -1 ;
                $data['notif2']        = -1 ;
                $data['notif3']        = -1 ;
                $data['form']         = $selfie->form ;
                $data['format']         = $selfie->format ;
                $data['date']         = date('Y-m-d H:i:s') ;
                $data['sync']         = 0 ;

                $id_photo = $db->insert('phocabine_photos', $data) ;

                file_get_contents(SITE . 'scripts/get_selfies.php?id_selfie=' . $selfie->id_selfie) ;
            }
        }

        echo 1 ;

        exit ;
    }
}

?>
